Option Explicit On
Imports system
Imports System.Drawing
Imports System.Windows.Forms
Imports System.Net.Mail

Public Class Form1
    Private Shared myTimer As New System.Windows.Forms.Timer()
    Dim WithEvents comport As New IO.Ports.SerialPort

    Dim blackPen As New Pen(Color.Black, 3)
    Dim gridPen As New Pen(Color.DimGray, 1)
    Dim drawBrush As New SolidBrush(Color.Black)
    Dim drawFont As New Font("Arial", 8, FontStyle.Bold)

    Dim border As Rectangle
    Dim ChartBorder As Rectangle

    Dim xStep, yStep As Single
    Dim dTemp As Single
    Dim dTime As TimeSpan
    Dim deltaTime As Single
    Dim StartZeit, StopZeit As Date

    Dim decWidth As Integer = 40   'Breitenversatz fr Diagrammgitter
    Dim decHeight As Integer = 30    'Hhenversatz fr Diagrammgitter
    Dim Kennlinie(1) As PointF      'definiere Feld fr Kennlinien-Koordinaten
    Dim nullinie(1) As PointF       'definiere Feld fr Nullachse (2 Punkte)
    Dim meanLinie(1) As PointF      'Mittelwertlinie im Diagrammm
    Dim minLinie(1) As PointF       'Minimalwertlinie
    Dim maxLinie(1) As PointF       'Maximalwertlinie
    Dim Mittelwert As Single
    Dim Minwert As Single
    Dim Maxwert As Single
    Dim AlarmMax As Boolean = False
    Dim AlarmMin As Boolean = False

    Dim yNullPix As Single
    Dim Messwerte As New Dictionary(Of Date, Single)

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'Initialisierungswerte fr Com-Settings
        Dim Paritaet As Array
        Dim Handshake As Array
        AddHandler myTimer.Tick, AddressOf TimerEventProcessor

        Paritaet = System.Enum.GetNames(GetType(IO.Ports.Parity))
        Handshake = System.Enum.GetNames(GetType(IO.Ports.Handshake))
        GetSerialPortNames()
        ComportCombo.SelectedIndex = 0
        BaudrateCombo.SelectedIndex = 6
        For Each hand As String In Handshake
            FlusssteuerungCombo.Items.Add(hand)
        Next
        For Each par As String In Paritaet
            ParityCombo.Items.Add(par)
        Next
        ParityCombo.SelectedIndex = 0
        FlusssteuerungCombo.SelectedIndex = 0

        ' Initialisierungswerte fr Chart
        comboZeitAchsenFormat.Items.AddRange(New String() {"H:mm:ss,f", "H:mm:ss", "H:mm", "mm:ss,ff", "mm:ss", "ss,ff", "ss", "d.MMM H:mm"})
        comboZeitAchsenFormat.SelectedIndex = 1

        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint, True)
        TempMaxUpDown.Value = 80
        TempMinUpDown.Value = -20
        dtMaximum.Value = DateTime.Now.AddMinutes(10)
        dtMinimum.Value = DateTime.Now
        lblStartzeit.Text = String.Format("{0:d},   {0:T} Uhr ", DateTime.Now)
        lblStoppzeit.Text = String.Format("{0:d},   {0:T} Uhr ", DateTime.Now)

        'Initialisierung fr Alarm-Email
        panelAlarmMail.Enabled = False
        'EMail-Einstellungen aus Anwendugseinstellungen lesen
        tbxFrom.Text = My.Settings.Absender
        tbxSmtpServer.Text = My.Settings.SMTP
        tbxPasswort.Text = My.Settings.Passwort
        tbxTo.Text = My.Settings.Empfaenger
        tbxLoginName.Text = My.Settings.LoginName
        tbxPort.Text = My.Settings.Port
        cbxAsync.Checked = My.Settings.Async
        cbxSSL.Checked = My.Settings.SSL
        tbxBody.Text = My.Settings.MailBody
        numAlarmLevel1.Value = My.Settings.AlarmLevel1
        numAlarmLevel2.Value = My.Settings.AlarmLevel2
    End Sub
    Private Sub Panel1_Paint(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles Panel1.Paint
        ' wird bei jedem neuzeichnen des Graphenbereichs (Panel1) neu aufgerufen!
        Dim tempPen As New Pen(lblTempPenColor.BackColor, NumPenSize.Value)
        Dim MeanPen As New Pen(lblMeanPenColor.BackColor, nudMeanPenSize.Value)
        Dim MinmaxPen As New Pen(lblMinmaxPenColor.BackColor, numMinmaxPenSize.Value)
        Dim PointsBrush As New SolidBrush(lblPointsColor.BackColor)

        border = New Rectangle(0, 0, Panel1.Width, Panel1.Height)  'Panelgrenzen
        e.Graphics.SmoothingMode = Drawing2D.SmoothingMode.AntiAlias    'Pixel gltten!

        ChartBorder = border
        ChartBorder.Inflate(-decWidth, -decHeight)                       'Chartgrenze reduzieren wegen Skalen
        ChartBorder.Offset(0, -5)
        Dim xAchse() As PointF = {New Point(ChartBorder.Left, ChartBorder.Bottom), _
                                  New Point(ChartBorder.Right, ChartBorder.Bottom)}
        Dim yAchse() As PointF = {New Point(ChartBorder.Left, ChartBorder.Top), _
                                  New Point(ChartBorder.Left, ChartBorder.Bottom)}  'Array von Linienkoordinaten fr y-Achse

        e.Graphics.DrawLines(blackPen, yAchse)          'Y-Achse zeichnen

        Dim i As Single = TempMinUpDown.Value
        dTemp = (TempMaxUpDown.Value - TempMinUpDown.Value) / yScaleUpDown.Value
        yStep = ChartBorder.Height / yScaleUpDown.Value
        For n As Single = ChartBorder.Bottom To ChartBorder.Top Step -yStep
            ' Horizontales Grid zeichnen
            e.Graphics.DrawLine(gridPen, ChartBorder.Left, n, ChartBorder.Right, n)
            ' Temperaturskala eintragen
            e.Graphics.DrawString(String.Format("{0:0.##}", i), drawFont, drawBrush, border.Left + 5, n - 7) ' Y-Skala
            i = i + dTemp 'Laufvariable fr Beschriftung der y-Achse
        Next n

        ' Vertikales Grid zeichnen
        dTime = TimeSpan.FromTicks(((dtMaximum.Value.Subtract(dtMinimum.Value)).Ticks / xScaleUpDown.Value))  ' delta Wert fr Beschriftung
        xStep = ChartBorder.Width / xScaleUpDown.Value          ' delta Wert fr Skala

        Dim ti As Date
        ti = dtMinimum.Value
        Dim drawFormat As New StringFormat
        drawFormat.FormatFlags = StringFormatFlags.NoClip 'StringFormatFlags.DirectionVertical Or Or StringFormatFlags.DirectionRightToLeft

        For n As Single = ChartBorder.Left To ChartBorder.Right Step xStep
            e.Graphics.DrawLine(gridPen, n, ChartBorder.Top, n, ChartBorder.Bottom)
            ' Achsen-Beschriftung (Zeitachse)
            e.Graphics.DrawString(ti.ToString(comboZeitAchsenFormat.SelectedItem), drawFont, drawBrush, n - 15, ChartBorder.Bottom + 2, drawFormat) ' X-Skala
            ti = ti + dTime  ' Laufvariable fr Achsen-Beschriftung
        Next n

        'Nullinie berechnen
        berechne_Nullinie()
        nullinie(0) = New PointF(ChartBorder.Left, yNullPix)
        nullinie(1) = New PointF(ChartBorder.Right, yNullPix)
        'Nullinie einzeichnen
        If TempMinUpDown.Value < 0 Then
            'Temperatur-Nullinie einzeichnen ... 
            e.Graphics.DrawLines(blackPen, nullinie)
        Else ' ... oder X-Achse ganz unten
            e.Graphics.DrawLines(blackPen, xAchse)      'X- Achse zeichnen
        End If

        'Mittelwertlinie berechnen und anzeigen
        If cbMittelwert.Checked AndAlso Messwerte.Count > 0 Then
            Dim yMeanPix As Single

            Mittelwert = mean(Messwerte)
            yMeanPix = yNullPix - Mittelwert * ChartBorder.Height / (TempMaxUpDown.Value - TempMinUpDown.Value)
            meanLinie(0) = New PointF(ChartBorder.Left, yMeanPix)
            meanLinie(1) = New PointF(ChartBorder.Right, yMeanPix)
            'Mittelwertlinie einzeichnen
            MeanPen.Alignment = Drawing2D.PenAlignment.Center
            e.Graphics.DrawLines(MeanPen, meanLinie)
        End If

        'MinMax-Kurve berechnen und anzeigen
        If cbMinmax.Checked AndAlso Messwerte.Count > 0 Then
            Dim yMinPix, yMaxPix As Single

            Minwert = min(Messwerte)
            Maxwert = max(Messwerte)
            yMinPix = yNullPix - Minwert * ChartBorder.Height / (TempMaxUpDown.Value - TempMinUpDown.Value)
            yMaxPix = yNullPix - Maxwert * ChartBorder.Height / (TempMaxUpDown.Value - TempMinUpDown.Value)
            minLinie(0) = New PointF(ChartBorder.Left, yMinPix)
            minLinie(1) = New PointF(ChartBorder.Right, yMinPix)
            maxLinie(0) = New PointF(ChartBorder.Left, yMaxPix)
            maxLinie(1) = New PointF(ChartBorder.Right, yMaxPix)
            'Min- und Maxwertlinie einzeichnen
            MinmaxPen.Alignment = Drawing2D.PenAlignment.Center
            e.Graphics.DrawLines(MinmaxPen, minLinie)
            e.Graphics.DrawLines(MinmaxPen, maxLinie)
        End If

        'Kennlinie zeichnen
        tempPen.Alignment = Drawing2D.PenAlignment.Center
        Try
            If cbTempCurve.Checked And Kennlinie.Length > 1 Then
                If cbSmoothCurve.Checked Then
                    e.Graphics.DrawCurve(tempPen, Kennlinie)
                Else
                    e.Graphics.DrawLines(tempPen, Kennlinie)
                End If
            End If
            If cbShowPoints.Checked Then
                For i = 0 To Kennlinie.Length - 1
                    e.Graphics.FillRectangle(PointsBrush, Kennlinie(i).X - nudPointsPenSize.Value, _
                        Kennlinie(i).Y - nudPointsPenSize.Value, nudPointsPenSize.Value * 2, nudPointsPenSize.Value * 2)
                Next i
            End If
        Catch
        Finally
        End Try
    End Sub
    Private Sub berechne_Nullinie()
        yNullPix = ChartBorder.Top + ChartBorder.Height * TempMaxUpDown.Value / (TempMaxUpDown.Value - TempMinUpDown.Value)
    End Sub
    Private Sub KennlinienwerteSchreiben()
        Dim yPix, xPix As Single
        Dim yKonst As Single
        ReDim Kennlinie(Messwerte.Count - 1)
        Dim Messpunkt As KeyValuePair(Of Date, Single)

        Dim TimeMax As TimeSpan = dtMaximum.Value.Subtract(dtMinimum.Value)

        Try
            yKonst = ChartBorder.Height / (TempMaxUpDown.Value - TempMinUpDown.Value)
        Catch ex As Exception
            yKonst = 0
        End Try
        Dim i As Integer = 0
        For Each Messpunkt In Messwerte
            xPix = ChartBorder.Left + ((Messpunkt.Key.Ticks - dtMinimum.Value.Ticks) / TimeMax.Ticks) * ChartBorder.Width
            yPix = yNullPix - Messpunkt.Value * yKonst
            Kennlinie(i) = New PointF(xPix, yPix)
            i += 1
        Next
    End Sub
    Private Sub ReadSerialData()
        Dim readbuffer(0) As Byte           'Empfangspuffer fr ein Zeichen (8Bit-Tempwert)
        Dim messwert As Single              'Intern werden die Messwerte als Single verarbeitet!

        Try
            If comport.IsOpen Then comport.Close()
            ' Comport Settings
            comport.Handshake = FlusssteuerungCombo.SelectedIndex
            comport.Parity = ParityCombo.SelectedIndex
            comport.StopBits = StoppbitsNumeric.Value
            comport.BaudRate = BaudrateCombo.SelectedItem
            comport.DataBits = DatenbitsNumeric.Value
            comport.PortName = ComportCombo.Text
            comport.WriteTimeout = deltaTime * 500000   ' Timeout nach Intervallzeit/2
            comport.ReadTimeout = deltaTime * 500000

            comport.Open()
            Dim messtemp As Single = 0
            For i As Integer = 1 To numAnzahlWerte.Value
                comport.Write(Chr(2))                       ' STX (Start Of Text) senden!
                comport.Read(readbuffer, 0, 1)              ' Temperaturwert einlesen
                If readbuffer(0) > 127 Then                 ' Negative Temperatur??
                    messtemp += CSng(-(256 - readbuffer(0))) '   dann: Umwandlung 2er-Komplemet in Single
                Else                                        '         mit Vorzeichen
                    messtemp += CSng(readbuffer(0))          '   sonst: Konvertieren in Single
                End If
            Next i
            messwert = messtemp / numAnzahlWerte.Value

            tbMesswert.Text = messwert.ToString

            StopZeit = DateTime.Now
            Messwerte.Add(StopZeit, messwert)           'Messwert mit aktuellem Datum/Zeit speichern
            If Messwerte.Count > 0 Then                 ' Kennlinie nur schreiben, wenn mehr als 1 MW
                KennlinienwerteSchreiben()
            End If

            If cbAlarmMailSenden.Checked Then           ' Bedingungen fr Mail-Alarmierung prfen!
                If cbTemp1Over.Checked Then
                    If Not AlarmMax AndAlso messwert > numAlarmLevel1.Value Then
                        AlarmMax = True
                        sendMail("Level1Over", StopZeit, messwert)
                    ElseIf AlarmMax AndAlso messwert <= numAlarmLevel1.Value Then
                        If cbTemp1Under.Checked Then sendMail("Level1Under", StopZeit, messwert)
                        AlarmMax = False
                    End If
                End If
                If cbTemp2Under.Checked Then
                    If Not AlarmMin AndAlso messwert < numAlarmLevel2.Value Then
                        AlarmMin = True
                        sendMail("Level2Under", StopZeit, messwert)
                    ElseIf AlarmMin AndAlso messwert >= numAlarmLevel2.Value Then
                        If cbTemp2Under.Checked Then sendMail("Level2Over", StopZeit, messwert)
                        AlarmMin = False
                    End If
                End If
            End If

            lblMesspunkte.Text = lblMesspunkte.Text + 1
            lblStoppzeit.Text = String.Format("{0:d},   {0:T} Uhr ", StopZeit)

            tbMin.Text = min(Messwerte).ToString
            tbMax.Text = max(Messwerte).ToString
            tbMean.Text = mean(Messwerte).ToString

            Panel1.Refresh()                            ' Kennlinie neu zeichnen

        Catch IOerr As IO.IOException
            btnStopp.PerformClick()
            MsgBox("IO-error:" + Chr(13) + Chr(10) + IOerr.Message + _
                        Chr(13) + Chr(10) + "Die Meswertaufnahme wird beendet!", MsgBoxStyle.Exclamation)
            'lblReadError.Text = lblReadError.Text + 1
            Return
        Catch IOerr As System.Exception
            lblReadError.Text = lblReadError.Text + 1
            'Threading.Thread.Sleep(100) '100ms warten
            'ReadSerialData()            ' neuer Versuch (rekursiv!!)
            Return
        Finally
            If comport.IsOpen Then comport.Close()
        End Try
    End Sub
    Private Sub TimerEventProcessor(ByVal myObject As Object, ByVal myEventArgs As EventArgs)
        ReadSerialData()            ' Messwert einlesen
    End Sub
    Private Sub setTimerIntervall()        'Timer Intervall setzen
        deltaTime = nudIntervall.Value
        Select Case cbIntervallUnit.Text
            Case "sek"
            Case "min"
                deltaTime *= 60
            Case "std"
                deltaTime *= 3600
        End Select                      ' deltaTime in Sekunden
        myTimer.Interval = CInt(deltaTime * 1000)       ' Timer Intervall in ms bergeben!!!
    End Sub
    '##################################################
    ' Sonstige Funktionen
    Private Function min(ByVal a1 As Dictionary(Of Date, Single)) As Single
        Dim entry As KeyValuePair(Of Date, Single)
        Dim i As Single
        For Each entry In a1
            If i = Nothing Then
                i = entry.Value
            Else
                If entry.Value < i Then i = entry.Value
            End If
        Next
        Return i
    End Function
    Private Function max(ByVal a1 As Dictionary(Of Date, Single)) As Single
        Dim entry As KeyValuePair(Of Date, Single)
        Dim i As Single
        For Each entry In a1
            If i = Nothing Then
                i = entry.Value
            Else
                If entry.Value > i Then i = entry.Value
            End If
        Next
        Return i
    End Function
    Private Function mean(ByVal a1 As Dictionary(Of Date, Single)) As Single
        Dim entry As KeyValuePair(Of Date, Single)
        Dim i As Single = 0
        Dim n As Integer
        For Each entry In a1
            i += entry.Value
            n += 1
        Next
        i /= n
        Return i
    End Function
    '#######Messen#####################################
    'Aufnahme-Steuerung (Schaltflchen)
    Private Sub btnAufnahme_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAufnahme.Click
        Messwerte.Clear()
        setTimerIntervall()
        myTimer.Start()

        lblMesspunkte.Text = 0
        lblReadError.Text = 0
        btnPause.Enabled = True

        'Initialisieren der Fortschrittsanzeige
        pbSampleTemp.Enabled = True
        pbSampleTemp.Maximum = 10
        pbSampleTemp.Value = 0
        pbSampleTemp.Step = 1
        pbSampleTemp.MarqueeAnimationSpeed = 100

        'Anzeigen neu initialisieren
        StartZeit = DateTime.Now
        StopZeit = DateTime.Now
        lblStartzeit.Text = String.Format("{0:d},   {0:T} Uhr ", StartZeit)
        lblStoppzeit.Text = String.Format("{0:d},   {0:T} Uhr ", StopZeit)
        dtMinimum.Value = StartZeit
        dtMaximum.Value = DateTime.Now.AddMinutes(10)
        tbMin.Text = ""
        tbMax.Text = ""
        tbMean.Text = ""
        tbMesswert.Text = ""

        ReadSerialData()
    End Sub
    Private Sub btnStopp_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnStopp.Click
        myTimer.Stop()
        pbSampleTemp.MarqueeAnimationSpeed = 0
        If btnPause.Text = "Weiter" Then btnPause.Text = "Pause"
        btnPause.Enabled = False
        btnAufnahme.Enabled = True
    End Sub
    Private Sub btnPause_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPause.Click
        If btnPause.Text = "Pause" Then
            myTimer.Stop()
            pbSampleTemp.MarqueeAnimationSpeed = 0
            btnPause.Text = "Weiter"
            btnAufnahme.Enabled = False
            pbSampleTemp.Enabled = False
        Else
            setTimerIntervall()         'Timer Intervall kann nach Pause gendert werden!
            myTimer.Start()
            pbSampleTemp.MarqueeAnimationSpeed = 100
            pbSampleTemp.Enabled = True
            btnPause.Text = "Pause"
            btnAufnahme.Enabled = True
        End If
    End Sub
    Private Sub numIntervall_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles nudIntervall.ValueChanged, numAnzahlWerte.ValueChanged
        If nudIntervall.Value >= 60 Then
            Select Case cbIntervallUnit.Text
                Case "sek"
                    cbIntervallUnit.SelectedIndex = 1
                    nudIntervall.Value = 1
                Case "min"
                    cbIntervallUnit.SelectedIndex = 2
                    nudIntervall.Value = 1
                Case "std"
            End Select
        ElseIf nudIntervall.Value < 1 Then
            Select Case cbIntervallUnit.Text
                Case "sek"
                    nudIntervall.Value = 1
                Case "min"
                    cbIntervallUnit.SelectedIndex = 0
                    nudIntervall.Value = 59
                Case "std"
                    cbIntervallUnit.SelectedIndex = 1
                    nudIntervall.Value = 59
            End Select
        End If
    End Sub
    '#######Einstellungen##############################
    'Raster
    Private Sub RasterAnzahl_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles yScaleUpDown.ValueChanged, xScaleUpDown.ValueChanged
        Panel1.Refresh()
    End Sub
    'Temperaturachse
    Private Sub TempMaxUpDown_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TempMaxUpDown.ValueChanged
        If TempMaxUpDown.Value <= TempMinUpDown.Value Then
            TempMaxUpDown.Value = TempMinUpDown.Value + 1
        End If
        berechne_Nullinie()
        KennlinienwerteSchreiben()
        Panel1.Refresh()
    End Sub
    Private Sub TempMinUpDown_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TempMinUpDown.ValueChanged
        If TempMinUpDown.Value >= TempMaxUpDown.Value Then
            TempMinUpDown.Value = TempMaxUpDown.Value - 1
        End If
        berechne_Nullinie()
        KennlinienwerteSchreiben()
        Panel1.Refresh()
    End Sub
    'Zeitachsensteuerung
    Private Sub ZeitAchsen_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles dtMaximum.ValueChanged, dtMinimum.ValueChanged
        KennlinienwerteSchreiben()
        Panel1.Refresh()
    End Sub
    Private Sub ZeitAchsenFormat_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles comboZeitAchsenFormat.SelectedIndexChanged
        Panel1.Refresh()
    End Sub
    Private Sub btnTimeShiftRight_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles btnTimeShiftRight.MouseDown
        Dim ShiftStep As TimeSpan

        ShiftStep = TimeSpan.FromTicks((dtMaximum.Value.Ticks - dtMinimum.Value.Ticks) / 20)
        dtMaximum.Value += ShiftStep
        dtMinimum.Value += ShiftStep
    End Sub
    Private Sub btnTimeShiftLeft_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles btnTimeShiftLeft.MouseDown
        Dim ShiftStep As TimeSpan

        ShiftStep = TimeSpan.FromTicks((dtMaximum.Value.Ticks - dtMinimum.Value.Ticks) / 20)
        dtMaximum.Value -= ShiftStep
        dtMinimum.Value -= ShiftStep
    End Sub
    Private Sub btnTimeShiftLeftGross_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles btnTimeShiftLeftGross.MouseDown
        Dim ShiftStep As TimeSpan

        ShiftStep = TimeSpan.FromTicks((dtMaximum.Value.Ticks - dtMinimum.Value.Ticks) / 2)
        dtMaximum.Value -= ShiftStep
        dtMinimum.Value -= ShiftStep
    End Sub
    Private Sub btnTimeShiftRightGross_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles btnTimeShiftRightGross.MouseDown
        Dim ShiftStep As TimeSpan

        ShiftStep = TimeSpan.FromTicks((dtMaximum.Value.Ticks - dtMinimum.Value.Ticks) / 2)
        dtMaximum.Value += ShiftStep
        dtMinimum.Value += ShiftStep
    End Sub
    Private Sub btnZoomIn_MouseDown(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomIn.MouseDown
        Dim ShiftStep As TimeSpan

        ShiftStep = TimeSpan.FromTicks((dtMaximum.Value.Ticks - dtMinimum.Value.Ticks) / 20)
        dtMaximum.Value -= ShiftStep
        dtMinimum.Value += ShiftStep
    End Sub
    Private Sub btnZoomOut_MouseDown(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnZoomOut.MouseDown
        Dim ShiftStep As TimeSpan

        ShiftStep = TimeSpan.FromTicks((dtMaximum.Value.Ticks - dtMinimum.Value.Ticks) / 20)
        dtMaximum.Value += ShiftStep
        dtMinimum.Value -= ShiftStep
    End Sub
    'Kurven anzeigen und Stiftfarben und -Breiten
    Private Sub cbSmoothCurve_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbSmoothCurve.CheckedChanged, cbTempCurve.CheckedChanged, cbMinmax.CheckedChanged, cbMittelwert.CheckedChanged, cbShowPoints.CheckedChanged
        Panel1.Refresh()
    End Sub
    Private Sub FarbenColor_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lblPointsColor.Click, lblMeanPenColor.Click, lblTempPenColor.Click, lblMinmaxPenColor.Click
        ColorDialog.Color = sender.Backcolor
        ColorDialog.ShowDialog()
        sender.BackColor = ColorDialog.Color
        Panel1.Refresh()
    End Sub
    Private Sub numPenSize_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles nudMeanPenSize.ValueChanged, nudPointsPenSize.ValueChanged, numMinmaxPenSize.ValueChanged, NumPenSize.ValueChanged
        Panel1.Refresh()
    End Sub
    '########Com Settings###############################
    Private Sub btnSearchComPort_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSearchComPort.Click
        GetSerialPortNames()
    End Sub
    Sub GetSerialPortNames()
        ComportCombo.Items.Clear()
        ComportCombo.Sorted = True

        For Each sp As String In My.Computer.Ports.SerialPortNames
            ComportCombo.Items.Add(sp)
        Next
    End Sub
    '########Datei######################################
    'Datei als CSV speichern oder laden
    Private Sub btnSpeichern_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSpeichern.Click
        Dim saveFileDialog1 As New SaveFileDialog()
        saveFileDialog1.Filter = "CSV-Datei|*.csv"
        saveFileDialog1.FileName = "Temperatur"
        saveFileDialog1.Title = "Daten als CSV-Datei speichern"
        saveFileDialog1.ShowDialog()

        If saveFileDialog1.FileName <> "" Then
            Dim entry As KeyValuePair(Of Date, Single)
            Dim sw As IO.StreamWriter = IO.File.CreateText(saveFileDialog1.FileName)
            Dim chTrenn As Char

            If rbtTrennTab.Checked Then
                chTrenn = vbTab
            ElseIf rbtTrennKomma.Checked Then
                chTrenn = ","
            ElseIf rbtTrennSemikolon.Checked Then
                chTrenn = ";"
            ElseIf rbtTrennSonstige.Checked Then
                chTrenn = tbxTrenn.Text
            End If

            If cbInfoHeader.Checked Then        'Info-Header in Datei schreiben
                sw.WriteLine(tbxChartTitle.Text)
                sw.WriteLine("Messbeginn:" + chTrenn + String.Format("{0:d}  {0:T}", StartZeit))
                sw.WriteLine("Messende:" + chTrenn + String.Format("{0:d}  {0:T}", StopZeit))
                sw.WriteLine("Uhrzeit" + chTrenn + "Temperatur in C")
            End If
            For Each entry In Messwerte
                If cbSaveWithDate.Checked Then
                    sw.WriteLine(entry.Key.ToShortDateString + " " + entry.Key.ToLongTimeString + chTrenn + entry.Value.ToString)
                Else
                    sw.WriteLine(entry.Key.ToLongTimeString + chTrenn + entry.Value.ToString)
                End If
            Next
            sw.Close()
        End If
    End Sub
    Private Sub btnLaden_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnLaden.Click
        Dim openFileDialog1 As New OpenFileDialog()
        Dim Spalten As String()
        Dim parser As Microsoft.VisualBasic.FileIO.TextFieldParser

        openFileDialog1.Filter = "CSV-Datei|*.csv"
        openFileDialog1.Title = "Daten aus CSV-Datei lesen"

        If openFileDialog1.ShowDialog() = DialogResult.OK Then
            Dim chTrenn As Char

            If rbtTrennTab.Checked Then
                chTrenn = vbTab
            ElseIf rbtTrennKomma.Checked Then
                chTrenn = ","
            ElseIf rbtTrennSemikolon.Checked Then
                chTrenn = ";"
            ElseIf rbtTrennSonstige.Checked Then
                chTrenn = tbxTrenn.Text
            End If

            Messwerte.Clear() ' eventuell alte Messwerte lschen
            Try
                parser = My.Computer.FileSystem.OpenTextFieldParser(openFileDialog1.FileName, New String() {chTrenn})

                If cbInfoHeader.Checked Then        'Info-Header aus Datei lesen
                    Spalten = parser.ReadFields()   ' Erste Zeile enthlt Info-Header
                    tbxChartTitle.Text = Spalten(0)
                    Spalten = parser.ReadFields()   ' zweite Zeile enthlt Start-Zeit
                    StartZeit = Spalten(1)
                    Spalten = parser.ReadFields()   ' dritte Zeile enthlt Stopp-Zeit
                    StopZeit = Spalten(1)
                    Spalten = parser.ReadFields()   ' Dummy-Read fr Spalten-berschriften
                End If
            Catch ex As Microsoft.VisualBasic.FileIO.MalformedLineException
                MsgBox("Header hat ein ungltiges Format", MsgBoxStyle.Exclamation)
                Return
            Catch err As System.Exception
                MsgBox("Allgemeiner Fehler" & vbNewLine & err.Message, MsgBoxStyle.Exclamation)
                Return
            End Try

            While Not parser.EndOfData      ' Einlesen der Daten bis Dateiende
                Try
                    Spalten = parser.ReadFields()
                    Messwerte.Add(Spalten(0), Spalten(1))
                Catch ex As Microsoft.VisualBasic.FileIO.MalformedLineException
                    MsgBox("Tabelle hat ein ungltiges Format", MsgBoxStyle.Exclamation)
                    Return
                Catch err As System.Exception
                    MsgBox(err.Message, MsgBoxStyle.Exclamation)
                    Return
                End Try
            End While
            KennlinienwerteSchreiben()

            If Not cbInfoHeader.Checked Then    'Start und Stoppzeit aus den Messwerten lesen
                Dim dtMess(Messwerte.Count - 1) As Date
                Messwerte.Keys.CopyTo(dtMess, 0)    'Datum/Zeit (Schlssel) aller Messwerte in dtMess kopieren
                StartZeit = dtMess(0).ToString
                StopZeit = dtMess(Messwerte.Count - 1).ToString
            End If

            dtMaximum.Value = StopZeit
            dtMinimum.Value = StartZeit
            lblStartzeit.Text = String.Format("{0:d},   {0:T} Uhr ", StartZeit)
            lblStoppzeit.Text = String.Format("{0:d},   {0:T} Uhr ", StopZeit)
            tbMax.Text = max(Messwerte).ToString
            tbMin.Text = min(Messwerte).ToString
            tbMean.Text = mean(Messwerte).ToString
            tbMesswert.Text = ""
            lblMesspunkte.Text = Messwerte.Count.ToString

        End If
    End Sub
    Private Sub rbtTrennSonstige_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles rbtTrennSonstige.CheckedChanged
        If Not tbxTrenn.Enabled Then
            tbxTrenn.Enabled = True
        Else
            tbxTrenn.Enabled = False
        End If
    End Sub
    '########Alarm######################################
    Private Sub cbAlarmMailSenden_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbAlarmMailSenden.CheckedChanged
        panelAlarmMail.Enabled = Not panelAlarmMail.Enabled
    End Sub
    Private Sub sendMail(ByVal mode As String, Optional ByVal Zeit As Date = #1/1/2008#, Optional ByVal Temperatur As Single = 0)
        Try
            Dim smtp As SmtpClient
            Dim msg As MailMessage

            Dim FromAddress As MailAddress = New MailAddress(tbxFrom.Text)
            Dim ToAddress As MailAddress = New MailAddress(tbxTo.Text)

            msg = New MailMessage(FromAddress, ToAddress)
            Select Case mode
                Case "Level1Over"
                    msg.Subject = "ComMeldung: Warnung, Maximaltemperatur von " + Format(numAlarmLevel1.Value, "f") + "C berschritten!"
                    msg.Body = _
                    "Uhrzeit:    " + String.Format("{0:d},   {0:T} Uhr ", Zeit) + vbCrLf + _
                    "Temperatur: " + String.Format("{0:f}C ", Temperatur) + vbCrLf + _
                    tbxBody.Text
                Case "Level1Under"
                    msg.Subject = "ComMeldung: Normaltemperaturbereich wieder erreicht!"
                    msg.Body = _
                    "Uhrzeit:    " + String.Format("{0:d},   {0:T} Uhr ", Zeit) + vbCrLf + _
                    "Temperatur: " + String.Format("{0:f}C ", Temperatur) + vbCrLf + _
                    tbxBody.Text
                Case "Level2Over"
                    msg.Subject = "ComMeldung: Normaltemperaturbereich wieder erreicht!"
                    msg.Body = _
                    "Uhrzeit:    " + String.Format("{0:d},   {0:T} Uhr ", Zeit) + vbCrLf + _
                    "Temperatur: " + String.Format("{0:f}C ", Temperatur) + vbCrLf + _
                    tbxBody.Text
                Case "Level2Under"
                    msg.Subject = "ComMeldung: Warnung, Minimaltemperatur von " + Format(numAlarmLevel2.Value, "f") + "C unterschritten!"
                    msg.Body = _
                    "Uhrzeit:    " + String.Format("{0:d},   {0:T} Uhr ", Zeit) + vbCrLf + _
                    "Temperatur: " + String.Format("{0:f}C ", Temperatur) + vbCrLf + _
                    tbxBody.Text
                Case "Test"
                    msg.Subject = "ComMeldung: Alarmierungsmail Smtp-Test!!"
                    msg.Body = "Dies ist eine Testmail!" + vbCrLf + _
                    "Bitte lschen Sie die Mail sofort nach erhalt!"
            End Select

            smtp = New SmtpClient(tbxSmtpServer.Text, tbxPort.Text)  ' Postausgang spezifizieren
            smtp.DeliveryMethod = SmtpDeliveryMethod.Network    ' EMail ber LAN versenden

            If cbxSSL.Checked Then
                smtp.EnableSsl = True
            Else
                smtp.EnableSsl = False
            End If
            ' Anmeldeinformationen: Loginname und Passwort
            smtp.Credentials = New System.Net.NetworkCredential(tbxLoginName.Text, tbxPasswort.Text)

            If Not cbxAsync.Checked Then
                smtp.Timeout = 5000  ' Timeout fr synchrones senden = 5s
                smtp.Send(msg)  'E-Mail synchron versenden (Thread wird angehalten!)
            Else                'Email Asynchron versenden (Thread wird fortgesetzt!)
                AddHandler smtp.SendCompleted, AddressOf smtp_sent  'Ereignisprozedur einrichten
                smtp.SendAsync(msg, msg.Subject + " an " + msg.To(0).Address)
            End If

        Catch ex As Exception
            MsgBox("Send Mail fehlgeschlagen!" + vbCrLf + ex.Message.ToString, MsgBoxStyle.Exclamation)
        End Try
    End Sub
    Private Sub smtp_sent(ByVal sender As Object, ByVal e As System.ComponentModel.AsyncCompletedEventArgs)
        'Meldung wenn Nachricht asynchron versendet wurde!
        MsgBox("Nachricht: " + vbNewLine + e.UserState.ToString + vbNewLine + " erfolgreich versandt!")
    End Sub
    Private Sub btnMailTest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMailTest.Click
        sendMail("Test")
    End Sub
    Private Sub btnSaveMailSettings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSaveMailSettings.Click
        My.Settings.Absender = tbxFrom.Text     ' EMail-Settings speichern
        My.Settings.SMTP = tbxSmtpServer.Text
        My.Settings.Passwort = tbxPasswort.Text
        My.Settings.Empfaenger = tbxTo.Text
        My.Settings.LoginName = tbxLoginName.Text
        My.Settings.Port = tbxPort.Text
        My.Settings.Async = cbxAsync.Checked
        My.Settings.SSL = cbxSSL.Checked
        My.Settings.MailBody = tbxBody.Text
        My.Settings.AlarmLevel1 = numAlarmLevel1.Value
        My.Settings.AlarmLevel2 = numAlarmLevel2.Value
        My.Settings.Save()              ' Settings in den Anwendungseigenschaften speichern
    End Sub
    '#########Allgemein#################################
    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.Close()
    End Sub
    Private Sub LinkLabel1_LinkClicked(ByVal sender As System.Object, ByVal e As System.Windows.Forms.LinkLabelLinkClickedEventArgs) Handles LinkLabel1.LinkClicked
        Process.Start("http://mikrocontroller.rahm-home.de/Projekte.htm")
    End Sub
    Private Sub btnPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPrint.Click
        Dim pf As New Microsoft.VisualBasic.PowerPacks.Printing.PrintForm
        pf.Form = Me
        MsgBox("Stellen Sie die das Seitenlayout auf Querformat ein!" + vbCrLf + _
               "Start -> Drucker und Faxgerte -> Standarddrucker[Druckeinstellungen..] -> Querfomat", MsgBoxStyle.Information)

        Me.Refresh()    'Sonst sind Artefakte auf dem Ausdruck sichtbar!
        pf.PrintAction = Printing.PrintAction.PrintToPreview
        pf.Print()
    End Sub
    Private Sub tbxChartTitle_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles tbxChartTitle.MouseClick
        If tbxChartTitle.Text = "Chart Titel hier eingeben" Then
            tbxChartTitle.Text = ""
        End If
    End Sub
End Class
